import threading

from manganelo.api import SearchManga


class SearchMangaThread(SearchManga):
    def __init__(self, title: str):
        super().__init__(title)

        self._thread = threading.Thread(target=super().start)

    def start(self):
        self._thread.start()

    def wait(self):
        while self._thread.is_alive():
            pass

    def done(self):
        return not self._thread.is_alive()

    def __enter__(self):
        self.start()
        return self






from manganelo.api import SearchManga
from manganelo.api import MangaInfo
from manganelo.api import DownloadChapter

from manganelo import api_

if __name__ == "__main__":
	search = api_.SearchMangaThread("Naruto")

	search.start()

	# do stuff here while we search in the background

	search.wait()

	for r in search:
		print(r)


import requests
import typing

from bs4 import BeautifulSoup


class APIBase:
	_SEARCH_URL = "http://manganelo.com/search/"

	@staticmethod
	def _get_soup(url: str) -> typing.Union[BeautifulSoup, None]:
		r = APIBase._send_request(url)

		if r.status_code == requests.codes.ok:
			return BeautifulSoup(r.content, "html.parser")

	@staticmethod
	def _send_request(url: str):
		default_headers = requests.utils.default_headers()

		r = requests.get(url, stream=True, timeout=5, headers=default_headers)

		r.raise_for_status()

		if r.status_code == requests.codes.ok:
			return r


import os
import shutil
import tempfile
import typing

from PIL import Image
from reportlab.pdfgen import canvas

from manganelo.api import APIBase


class DownloadChapter(APIBase):
	def __init__(self, src_url: str, dst_path: str):
		super().__init__()

		self.ok = False

		self._src_url = src_url
		self._dst_path = dst_path

		self._download_chapter()

	def _download_chapter(self):
		soup = self._get_soup(self._src_url)

		if soup is None:
			raise Exception("Request failed")

		image_urls = self._get_image_urls(soup)

		with tempfile.TemporaryDirectory() as temp_dir:
			image_paths = self._download_images(image_urls, temp_dir)

			self._create_pdf(image_paths)

	@staticmethod
	def _get_image_urls(soup) -> typing.List[str]:
		image_soup = soup.find_all("img")

		image_urls = [ele["src"] for ele in image_soup]

		return image_urls

	def _create_pdf(self, images: typing.List[str]):
		pdf = canvas.Canvas(self._dst_path)

		for image in images:
			try:
				with Image.open(image) as img:
					w, h = img.size

			except (OSError, UnboundLocalError):
				continue

			pdf.setPageSize((w, h))

			try:
				pdf.drawImage(image, x=0, y=0)

			except OSError as e:
				continue

			pdf.showPage()

		pdf.save()

		self.ok = True

	def _download_images(self, image_urls: typing.List[str], save_dir: str) -> typing.List[str]:
		image_paths = []

		for i, url in enumerate(image_urls):
			image = self._send_request(url)
			image_ext = url.split(".")[-1]
			image_dst_path = os.path.join(save_dir, f"{i}.{image_ext}")

			if image is not None:
				# Download the image URL
				with open(image_dst_path, "wb") as fh:
					image.raw.decode_content = True

					try:
						shutil.copyfileobj(image.raw, fh)
					except:  # Catch everything and call it a day
						pass
					else:
						image_paths.append(image_dst_path)

		return image_paths




from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="manganelo",
    packages=find_packages(),
    version="1.4.0",
    license="MIT",

    description="Unofficial API for the Manganelo (and Mangakakalot) website.",
    long_description=long_description,
    long_description_content_type="text/markdown",

    author="Joshua Nixon",
    author_email="nixonjoshua98@gmail.com",

    url="https://github.com/nixonjoshua98/manganelo",

    download_url="https://github.com/nixonjoshua98/manganelo/archive/v1.4.0.tar.gz",
  
    keywords=["manga", "manganelo", "scrapper", "web", "mangakakalot", "thread"],
  
    install_requires=[
        "bs4",
        "requests",
        "reportlab",
    ],

    classifiers=[
      "Programming Language :: Python :: 3",
      "License :: OSI Approved :: MIT License",
      "Operating System :: OS Independent",
      "Development Status :: 5 - Production/Stable",
      "Intended Audience :: Developers",
      "Topic :: Software Development :: Build Tools",
    ],

    python_requires='>=3.7'
)



import dataclasses
import ast

from manganelo.api import APIBase


@dataclasses.dataclass
class MangaChapter:
	url: str
	chapter_num: float


class MangaInfo(dict, APIBase):
	def __init__(self, url: str):
		super().__init__()

		self._url = url

		self._page_soup = self._get_soup(url)

		self._parse_info()

	def __enter__(self):
		return self

	def __exit__(self, exc_type, exc_val, exc_tb):
		""" Context manager exit point """

	def _parse_info(self):
		# Elements
		info_panel = self._page_soup.find(class_="panel-story-info")
		info_right = info_panel.find(class_="story-info-right")
		info_left = info_panel.find(class_="story-info-left")
		right_extend = info_panel.find(class_="story-info-right-extent")
		description = info_panel.find(class_="panel-story-info-description")
		image = info_left.find(class_="info-image")
		icon = image.find("img")

		try:
			last_updated_ele, views_ele, *_ = right_extend.find_all("span", class_="stre-value")
		except ValueError as e:
			raise ValueError(f"Some data could not be parsed")

		table = self._parse_table()

		# Data
		url = self._url
		authors = table["author"]
		status = table["status"]
		genres = table["genres"]
		alt_titles = table.get("alternative", [])
		chapters = self._get_chapter_list()
		views = int(views_ele.text.replace(",", ""))
		updated_on = last_updated_ele.text.strip() if last_updated_ele is not None else None
		title = info_right.find("h1").text.strip() if info_right is not None else None
		desc = description.text.replace("Description :", "").strip()
		icon_url = icon.get("src", None)

		self.update({
			"url": url, "authors": authors, "status": status, "genres": genres, "alt_titles": alt_titles,
			"chapters": chapters, "views": views, "updated_on": updated_on, "title": title, "desc": desc,
			"icon_url": icon_url
			})

	def _get_chapter_list(self):
		panels = self._page_soup.find(class_="panel-story-chapter-list")

		ls = []

		for i, ele in enumerate(reversed(panels.find_all(class_="a-h"))):
			if ele is not None:
				url = ele.find("a")["href"]
				chapter_num = ast.literal_eval(url.split("chapter_")[-1])
				chapter = MangaChapter(url=url, chapter_num=chapter_num)
				ls.append(chapter)

		return ls

	def _parse_table(self):
		table_section = self._page_soup.find(class_="variations-tableInfo")

		table_dict = {}

		if table_section is None:
			raise Exception(f"Table could not be found")

		for row in table_section.find_all("tr"):
			# Label amd the value elements
			lbl_ele, val_ele = row.find(class_="table-label"), row.find(class_="table-value")

			# e.g 'table-genres' to 'genres'
			key = lbl_ele.find("i").get("class")[0].split("-")[-1]

			val = None

			# Potential rows
			if key == "alternative":
				val = val_ele.text.strip().split(";")
			elif key == "author":
				val = val_ele.text.strip().split(",")
			elif key == "status":
				val = val_ele.text.strip()
			elif key == "genres":
				val = val_ele.text.strip().split("-")

			table_dict[key] = val

		return table_dict


import string
import requests

from dataclasses import dataclass
from bs4 import BeautifulSoup

from manganelo.errors import (TagNotFound, RequestFailed)


@dataclass
class MangaSearchResult:
	title: str
	url: str


class SearchManga:
	def __init__(self, title: str) -> None:
		self.results = []
		self.url = "http://manganelo.com/search/" + self._format_title(title)

	def start(self):
		response = self._send_request(self.url)

		if response is None:
			raise RequestFailed(f"Request failed")

		soup = BeautifulSoup(response.content, "html.parser")
		panels = soup.find(class_="panel-search-story")

		if panels is None:
			raise TagNotFound(f"Tag 'panel-search-story' not found in response HTML")

		stories = panels.find_all(class_="search-story-item")

		for i, ele in enumerate(stories):
			item = ele.find(class_="item-img")

			if item is None:
				raise TagNotFound(f"Tag 'item-img' not found in response HTML")

			title, link = item["title"], item["href"]

			manga_result = MangaSearchResult(title=title, url=link)

			self.results.append(manga_result)

	@staticmethod
	def _send_request(url):
		default_headers = requests.utils.default_headers()

		r = requests.get(url, stream=True, timeout=5, headers=default_headers)

		r.raise_for_status()

		if r.status_code == requests.codes.ok:
			return r

	@staticmethod
	def _format_title(title: str) -> str:
		allowed_characters: str = string.ascii_letters + string.digits + "_"

		return "".join([char.lower() for char in title.replace(" ", "_") if char in allowed_characters])

	def __enter__(self):
		self.start()

		return self

	def __exit__(self, exc_type, exc_val, exc_tb) -> None:
		pass

	def __len__(self) -> int:
		return len(self.results)

	def __getitem__(self, item):
		return self.results[item]

	def __contains__(self, item):
		return item in self.results

	def __iter__(self):
		return iter(self.results)
